#define BUILD_NUMBER \
	"build-3227872"
#define BUILD_NUMBER_NUMERIC \
	3227872
#define BUILD_NUMBER_NUMERIC_STRING \
	"3227872"
#define PRODUCT_BUILD_NUMBER \
	"product-build-52125"
#define PRODUCT_BUILD_NUMBER_NUMERIC \
	52125
#define PRODUCT_BUILD_NUMBER_NUMERIC_STRING \
	"52125"
