/*********************************************************
 * Copyright (C) 2011-2015 VMware, Inc. All rights reserved.
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation version 2 and no later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA
 *
 *********************************************************/

/*
 * vsock_version.h --
 *
 * Version definitions for the Linux vsock driver.
 */

#ifndef _VSOCK_VERSION_H_
#define _VSOCK_VERSION_H_

#define VSOCK_DRIVER_VERSION          9.8.1.0
#define VSOCK_DRIVER_VERSION_COMMAS   9,8,1,0
#define VSOCK_DRIVER_VERSION_STRING   "9.8.1.0"

#endif /* _VSOCK_VERSION_H_ */
